% resource /xsb/trailer.P

true(T,M) :- true2(T,M). % for dynamic assertions


% write a TRIPLE term to standard output
% used for showing variable bindings

write_triple(V) :- var(V), !, writeq(V).
write_triple(true(S,M)) :- !, write_triple(S), write('@'), write_triple(M).
write_triple(resource(NS,LN)) :- !, write_triple(NS), write(':'), write_triple(LN).
write_triple(triple(S,P,O)) :- !, write_triple(S),
  write('['), write_triple(P), write(','), write_triple(O), write(']').
write_triple((T1,T2)) :- !, write_triple(T1), write(', '), write_triple(T2).
write_triple(T) :- writeq(T).

% call an external program as a "model filter"

external_model(Program, IModel, OModel) :-
  IFile = '#input.model', OFile = '#output.model',
  open(IFile, write, IStream),
  namespaces(IModel, NSL),
  dump_rdf_header(IStream, NSL), !,
  dump_model(IStream, IModel, NSL),
  close(IStream),
  spawn_process([Program, IFile, OFile], _, _, _, Pid),
  process_control(Pid, wait),
  open(OFile, read, OStream),
  assert_process_output(OStream, OModel),
  close(OStream).

dump_model(Stream, Mdl, NSL) :-
  true(triple(S,P,O), Mdl),
  % writeln(triple(S,P,O)),
  dump_triple(Stream, triple(S,P,O), NSL),
  fail.
dump_model(Stream, _, _) :-
  writeln(Stream, '</rdf:RDF>').

% collect all namespaces as a set in NSL

namespaces(Mdl, NSL) :-
   namespaces1(Mdl, NSL1),
   namespaces2(Mdl, NSL2),
   namespaces3(Mdl, NSL3),
   merge(NSL1, NSL2, NSL12), !,
   merge(NSL12, NSL3, NSL), !.

namespaces1(Mdl, NSL) :- tsetof(N, ns1(N,Mdl), NSL), !.
namespaces1(_, []).
namespaces2(Mdl, NSL) :- tsetof(N, ns2(N,Mdl), NSL), !.
namespaces2(_, []).
namespaces3(Mdl, NSL) :- tsetof(N, ns3(N,Mdl), NSL), !.
namespaces3(_, []).

:- table ns1/2, ns2/2, ns3/2.

ns1(NS, Mdl) :- true(triple(resource(NS,_),_,_), Mdl).
ns2(NS, Mdl) :- true(triple(_,resource(NS,_),_), Mdl).
ns3(NS, Mdl) :- true(triple(_,_,resource(NS,_)), Mdl).

% dump standard rdf header (with entities)

dump_rdf_header(Stream, NSL) :-
  writeln(Stream, '<?xml version="1.0" encoding="ISO-8859-1"?>'),
  writeln(Stream, '<!DOCTYPE rdf:RDF ['),
  writeln(Stream, '  <!ENTITY rdf "http://www.w3.org/1999/02/22-rdf-syntax-ns#">'),
  dump_rdf_entities(Stream, NSL),
  writeln(Stream, ']>'),
  writeln(Stream, '<rdf:RDF xmlns:rdf="&rdf;"'),
  dump_rdf_xmlns(Stream, NSL),
  writeln(Stream, '>').

dump_rdf_entities(Stream, NSL) :-
  member(NS, NSL),
  write(Stream, '  <!ENTITY n'),
  ith(Index, NSL, NS),
  write(Stream, Index),
  write(Stream, ' "'),
  write(Stream, NS),
  writeln(Stream, '">'),
  fail.
dump_rdf_entities(_, _).

dump_rdf_xmlns(Stream, NSL) :-
  member(NS, NSL),
  write(Stream, '  xmlns:n'),
  ith(Index, NSL, NS),
  write(Stream, Index),
  write(Stream, '="&n'),
  write(Stream, Index),
  writeln(Stream, ';"'),
  fail.
dump_rdf_xmlns(_, _).

% dump a single triple as a single rdf:Description

dump_triple(Stream, triple(S,P,O), NSL) :-
  write(Stream, '<rdf:Description rdf:about='),
  dump_uri_string(Stream, S, NSL),
  writeln(Stream, '>'),
  dumpPO(Stream, P, O, NSL),
  writeln(Stream, '</rdf:Description>').

% dump predicate/object part (as one line)

dumpPO(Stream, P, resource(NS, LN), NSL) :- !,
  write(Stream, '  <'),
  dump_uri_tagname(Stream, P, NSL),
  write(Stream, ' rdf:resource='),
  dump_uri_string(Stream, resource(NS,LN), NSL),
  writeln(Stream, '/>').
dumpPO(Stream, P, O, NSL) :- % object is a literal
  write(Stream, '  <'),
  dump_uri_tagname(Stream, P, NSL),
  write(Stream, '>'),
  write(Stream, O),
  write(Stream, '</'),
  dump_uri_tagname(Stream, P, NSL),
  writeln(Stream, '>').

dump_uri_string(Stream, resource(NS,LN), NSL) :- !,
  write(Stream, '"&n'),
  ith(Index, NSL, NS),
  write(Stream, Index),
  write(Stream, ';'),
  write(Stream, LN),
  write(Stream, '"').
dump_uri_string(Stream, U, _) :-
  write('resource expected: '),
  writeln(U),
  write(Stream, '"#'),
  write(Stream, U),
  write(Stream, '"').

dump_uri_tagname(Stream, resource(NS,LN), NSL) :- !,
  ith(Index, NSL, NS),
  write(Stream, 'n'),
  write(Stream, Index),
  write(Stream, ':'),
  write(Stream, LN).
dump_uri_tagname(Stream, U, _) :-
  write('resource expected: '),
  writeln(U),
  write(Stream, U).


% read model from external program (i.e., from temp file)
% and assert all triples to true/2

assert_process_output(Stream, OModel) :-
  read(Stream, Triple),
  Triple \= end_of_file,
  assert(true2(Triple, OModel)),
  assert_process_output(Stream, OModel).
assert_process_output(_, _).

:- halt.

% end resource /xsb/trailer.P
