/*
 * Equal.java
 *
 * Created on February 18, 2002, 1:41 PM
 */
package org.semanticweb.dl;

import java.io.*;


/**
 * Equal axiom (for classes).
 * @author  Michael Sintek
 * @version 
 */
public class Equal extends Assertion {

  Expression _expr1, _expr2;
  
  /** Creates new Equal */
  public Equal(Expression expr1, Expression expr2) {
    _expr1 = expr1;
    _expr2 = expr2;
  }
  
  public void print (Syntax syntax, PrintWriter out) {
    if (syntax.XML ()) {
      out.print ("<EQUALC>");
      out.print ("<CONCEPT>");
      _expr1.print (syntax, out);
      out.print ("</CONCEPT>");
      out.print ("<CONCEPT>");
      _expr2.print (syntax, out);
      out.print ("</CONCEPT>");
      out.println ("</EQUALC>");
    } else if (syntax.TRIPLE()) {
      // triple(c1, daml:sameClassAs, c2).
      out.print("triple(");
      _expr1.print(syntax, out);
      out.print(",");
      out.print("resource('" + Symbol.DAML_NAMESPACE + "',sameClassAs)");
      out.print(",");
      _expr2.print(syntax, out);
      out.println(").");
    } else if (syntax.DOT()) {
      // do nothing, handled in Defconcept
    } else if (syntax.DAML()) {
      // <daml:Class about='expr2'/>
      // <daml:Class about='expr1'> <daml:sameClassAs resource='expr2'/> </>
      out.print("<daml:Class rdf:about='");
      _expr2.print(syntax, out);
      out.println("'/>");
      out.print("<daml:Class rdf:about='");
      _expr1.print(syntax, out);
      out.println("'>");
      out.print("  <daml:sameClassAs rdf:resource='");
      _expr2.print(syntax, out);
      out.println("'/>");
      out.println("</daml:Class>");
    } else {
      out.print ("(" + syntax.EQUAL () + " ");
      _expr1.print (syntax, out);
      out.print (" ");
      _expr2.print (syntax, out);
      out.println (")");
    }
  }
  
  public void toSHIQ(KB kb) {
    kb.add(new Equal(_expr1.toSHIQ(kb, CONCEPT), _expr2.toSHIQ(kb, CONCEPT)));
  }

}
