/*
 * Commandline.java
 *
 * Created on February 8, 2002, 9:57 AM
 */

package org.semanticweb.triple.application;

import java.io.*;
import org.semanticweb.triple.parser.*;
import org.semanticweb.triple.language.Clause;
import org.semanticweb.glp.*;
import org.semanticweb.lp.*;
import org.w3c.rdf.*;
import org.w3c.rdf.util.*;
import org.w3c.rdf.model.*;
import org.w3c.rdf.implementation.model.*;
import org.w3c.rdf.syntax.*;
import org.xml.sax.InputSource;

import com.hp.hpl.jena.rdf.arp.StanfordImpl;

/**
 * Process TRIPLE programs from command line.
 * @author  Michael Sintek
 * @version
 */
public class Commandline {
  
  final static String OUT = "#out.P";
  final static String XSBHEADER = "/xsb/header.P";
  final static String XSBTRAILER = "/xsb/trailer.P";
  
  public static void compileRDFFile (String filename, ExternalPrologCallback callback, String id) throws Exception{
    compileRDFStream (new FileInputStream (filename),filename, callback,id);
  }
  
  public static void compileRDFStream (InputStream rdfdstream,String ResourceName, ExternalPrologCallback callback, String id) throws Exception{
    //RDFParser parser = new RDFFactoryImpl().createParser(); //new com.hp.hpl.jena.rdf.arp.StanfordImpl();
    RDFParser parser = new com.hp.hpl.jena.rdf.arp.StanfordImpl ();
    InputSource input = new InputSource (rdfdstream);
    input.setSystemId (id);
    input.setPublicId (id);
    parser.parse (input, callback);
    //System.out.println("Loaded " + new Integer(count) + " Triples.");
  }

  public static void main(String args[]) {
    LTermParser parser = null;
    String rdfFileName = null;
    String rdfNamespaceString = null;
    String rdfNameString = null;
    int i=0;
    String inputFileName = null;

    if (args.length == 0) {
      parser = new LTermParser(System.in);
      inputFileName = "standard input";
    } else { // process arguments ... (output filename etc.) !!!
      if(args[i].equals("-rdfdata"))
        if ( i+3 < args.length) {
          rdfFileName = args[i+1];
          rdfNamespaceString = args[i+2];
          rdfNameString = args[i+3];
          i=i+4;
        }
      try {
        inputFileName = args[i];
        parser = new LTermParser(new java.io.FileInputStream(inputFileName));
      } catch (java.io.FileNotFoundException e) {
        System.out.println("Error: file '" + inputFileName + "' not found.");
        System.exit(1);
      }
    }
    try {
      PrintWriter writer = new PrintWriter(new FileWriter(OUT));
      writer.println("% " + OUT + ", generated from " + inputFileName + "\n");
      copyFile(writer, XSBHEADER);
      ExternalPrologCallback callback = new ExternalPrologCallback(writer);
      parser.parse(new Context(callback));
      if (rdfFileName != null) {
         callback.setRDFNamespace(new ResourceImpl(rdfNamespaceString,rdfNameString));
         compileRDFFile(rdfFileName,callback,rdfNamespaceString);
      }
      copyFile(writer, XSBTRAILER);
      writer.close();
      System.out.println("'" + OUT + "' written.");
    } catch (Exception e) {
      System.out.println("Error: " + e.getMessage());
      System.exit(1);
    }
  }
  
  static void copyFile(PrintWriter writer, String fileName) throws Exception {
    InputStream is = new Commandline().getClass().getResourceAsStream(fileName);
    if (is != null) {
      BufferedReader reader = new BufferedReader(new InputStreamReader(is));
      String line = reader.readLine();
      while (line != null) {
        writer.println(line);
        line = reader.readLine();
      }
      writer.println();
      reader.close();
    } else {
      System.err.println("resource '" + fileName + "' not found");
      System.err.println("[make sure java/resources is on your classpath]");
    }
  }
  
}
