package org.semanticweb.triple.inference;

/*
 *  Name: Body.java
 *  Version: 1.0
 *  Purpose: represents rule bodies
 *  History:
 *  7.7.1999 Added isNegated method SDe
 */
import java.io.*;
import java.util.*;
import java.net.*;

/**
 *  Description of the Class
 *
 *@author     Administrator
 *@created    January 17, 2002
 */
public class Body extends RuleAtom {
    // Rumpfatom
    boolean neg;
    // negiert oder nicht
    Atoms up1 = null;
    // fuer alternierenden Fixpunkt bei negativen Atomen
    Atoms up2 = null;
    // fuer alternierenden Fixpunkt bei negativen Atomen
    int bindex[];
    // the indices indicate the symbols of the variables, the content the columns of the resulting relation
    int dindex[];
    // number j at index i indicates that column i has to be copied to column j in the resulting relation
    Atoms hrelation = null;
    // Down- und Sidewayspropagierung
    Atoms addhrel = null;
    // Berechnung bei Negation
    boolean builtin = false;
    // gibt an, ob es sich um ein Builtin-Atom handelt
    int delays = 0;
    // Verzgerung der Auswertung bei Negation
    boolean delayed = false;
    int anzfilters = 0;

    Atoms brelation = null;
    // inkrementelle Berechnung des joins ber alle Rumfpatome
    TermSet termsets[];
    boolean changets[];
    TermSet btermsets[];
    boolean changebts[];



    /**
     *  Constructor for the Body object
     *
     *@param  sym      Description of the Parameter
     *@param  negated  Description of the Parameter
     *@param  terms    Description of the Parameter
     */
    public Body(int sym, boolean negated, Term[] terms) {
        // sym = Prdikatsymbol, anz = Stelligkeit, negated = negiert, terms = Feld von anz Termen
        super(sym, terms);
        neg = negated;
        up = new Atoms(anzvars);
        addup = new Atoms(anzvars);
        down = new Atoms(terms.length);
        adddown = new Atoms(terms.length);
        if (negated) {
            up1 = new Atoms(anzvars);
            up2 = new Atoms(anzvars);
        }
    }


    /**
     *  Description of the Method
     */
    public void ClearRuleAtom() {
        // Lschen der Zwischenergebnisse der Evaluierung
        super.ClearRuleAtom();
        if (up1 != null) {
            up1 = new Atoms(up1.arity);
        }
        if (up2 != null) {
            up2 = new Atoms(up2.arity);
        }
        if (brelation != null) {
            brelation = new Atoms(brelation.arity);
        }
        if (hrelation != null) {
            hrelation = new Atoms(hrelation.arity);
        }
        if (addhrel != null) {
            addhrel = new Atoms(addhrel.arity);
        }
    }


    /**
     *  Description of the Method
     */
    public void ClearRuleAtom1() {
        // Lschen der Zwischenergebnisse der Evaluierung
        if (!neg) {
            up = new Atoms(up.arity);
        }
        else {
            up1 = new Atoms(up1.arity);
        }
        down = new Atoms(down.arity);
        adddown = new Atoms(adddown.arity);
        addup = new Atoms(addup.arity);
        up = new Atoms(up.arity);
        if (brelation != null) {
            brelation = new Atoms(brelation.arity);
        }
        if (hrelation != null) {
            hrelation = new Atoms(hrelation.arity);
        }
        if (addhrel != null) {
            addhrel = new Atoms(addhrel.arity);
        }
    }


    /**
     *  Description of the Method
     *
     *@param  relation  Description of the Parameter
     *@param  index     Description of the Parameter
     *@return           Description of the Return Value
     */
    public boolean Down(Atoms relation, int index[]) {
        int oldnum;
        int newnum;
        // Downpropagierung ueber Rumpfatom
        oldnum = down.anztuples;
        down.Substitute(this, relation, index);
        newnum = down.anztuples;
        return oldnum != newnum;
    }


    /**
     *  Description of the Method
     *
     *@param  p  Description of the Parameter
     */
    public void internalize(PrintStream p) {
        if (neg) {
            p.print("-");
        }
        super.internalize(p);
    }


    /**
     *  Gets the negated attribute of the Body object
     *
     *@return    The negated value
     */
    public boolean isNegated() {
        // to to resort bodyatoms SDe
        return neg;
    }


    /**
     *  Description of the Method
     *
     *@param  brel   Description of the Parameter
     *@param  index  Description of the Parameter
     *@return        Description of the Return Value
     */
    public boolean NaiveRight(Atoms brel, int index[]) {
        int oldnum;
        int newnum;
        oldnum = brelation.anztuples;
        if (!neg) {
            // Join mit positivem Rumpfliteral
            brelation.Join(brel, index, up, index1, brel.matchindex, dindex);
        }
        else {
            // Differenz mit negativem Rumpfliteral
            brelation.Negation(brel, index, up, index1);
        }
        newnum = brelation.anztuples;
        return oldnum != newnum;
    }


    /**
     *  Description of the Method
     *
     *@param  p  Description of the Parameter
     */
    public void print(PrintStream p) {
        if (neg) {
            p.print('-');
        }
        super.print(p);
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public boolean Switch() {
        // schaltet die Fakten der negativen Literale um beim alternierenden Fixpunkt
        Atoms h;

        if (neg && !up1.Compare(up2)) {
            h = up2;
            up2 = up;
            up = up1;
            up1 = h;
            return true;
        }
        else {
            return false;
        }
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public String toString() {
        if (neg) {
            return "-" + super.toString();
        }
        else {
            return super.toString();
        }
    }
}
