package org.semanticweb.triple.inference;

/*
 *  Name: Variable.java
 *  Version: 1.2
 *  Purpose: realises Variables
 *  History: function internalize1 returning strings added (jan)
 *  interface simplified (jan)
 */
import java.io.*;
import java.util.*;
import java.net.*;

/**
 *  Description of the Class
 *
 *@author     Administrator
 *@created    January 17, 2002
 */
public class Variable extends Term {
    // type == 1
    /**
     *  Description of the Field
     */
    protected int symbol;
    // Variablensymbol
    /**
     *  Description of the Field
     */
    protected int hsymbol;
    // Feld zum Retten des Variablensymbols bei Umbenennungen
    /**
     *  Description of the Field
     */
    public Term subsby = null;
    // hier wird ein Term zur Substitution der Variablen angehngt
    /**
     *  Description of the Field
     */
    protected Variable next = null;

    // Verzeigerung der Variablen innerhalb eines Atoms


    /**
     *  Constructor for the Variable object
     *
     *@param  sym  Description of the Parameter
     */
    public Variable(int sym) {
        super(0, false);
        symbol = sym;
        ground = false;
        subsby = null;
        groundlevel = 1;
    }


    /**
     *  Description of the Method
     *
     *@param  t  Description of the Parameter
     */
    public void AttachSubs(Term t) {
        this.subsby = t;
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public Term Clone() {
        // kloniert den Term
        Variable v;
        v = new Variable(this.symbol);
        if (subsby != null) {
            v.subsby = subsby.Clone();
        }
        return v;
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public Term CloneFlat() {
        return new Variable(this.symbol);
    }


    /**
     *  Description of the Method
     *
     *@param  t  Description of the Parameter
     *@return    Description of the Return Value
     */
    public int CompareFlat(Term t) {
        // vergleicht zwei Variablen aufgrund ihrer Symbole
        return symbol <= ((Variable) t).symbol ? (symbol < ((Variable) t).symbol ? -1 : 0) : 1;
    }


    /**
     *  Gets the subs attribute of the Variable object
     *
     *@return    The subs value
     */
    public Term getSubs() {
        return this.subsby;
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public String InternalizeFlat() {
        return "X" + String.valueOf(symbol);
    }


    /**
     *  Gets the variable attribute of the Variable object
     *
     *@return    The variable value
     */
    public boolean isVariable() {
        return true;
    }


    /**
     *  Description of the Method
     *
     *@param  t2  Description of the Parameter
     *@return     Description of the Return Value
     */
    public boolean Match(Term t2) {
        if (this.subsby != null) {
            // first is a variable and is already substituted
            return this.subsby.Match(t2);
        }
        else if ((t2 instanceof Variable) && (((Variable) t2).subsby != null)) {
            this.subsby = ((Variable) t2).subsby;
            return true;
        }
        else {
            // Substitutiere Variable mit t2
            this.subsby = t2;
            return true;
        }
    }


    /**
     *  Description of the Method
     *
     *@param  v  Description of the Parameter
     *@return    Description of the Return Value
     */
    protected boolean occurs(Variable v) {
        // Occurence Check
        if (subsby != null) {
            return subsby.occurs(v);
        }
        else if (symbol == v.symbol) {
            return true;
        }
        else {
            return false;
        }
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public Term Substitute() {
        // Durchfhrung einer Substitution, d.h. die Variable wird durch ihren
        // Substituenden im Term ersetzt
        // Dabei werden alle oberen Terme (berhalb von Variablen) neu generiert
        if (subsby != null) {
            if (subsby.ground) {
                return subsby;
            }
            else {
                return subsby.Substitute();
            }
        }
        else {
            return new Variable(symbol);
        }
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public String toStringFlat() {
        return "X" + String.valueOf(symbol);
    }


    /**
     *  Description of the Method
     *
     *@param  p   Description of the Parameter
     *@param  f   Description of the Parameter
     *@param  st  Description of the Parameter
     *@return     Description of the Return Value
     */
    public String toStringFlat(String p[], String f[], String st[]) {
        // Ausgabe der Variablen mit dem Substitutenden als String,
        Character ch;
        String s;
        ch = new Character((char) ((int) (('X') + symbol)));
        s = ch.toString();
        if (subsby != null) {
            s = s.concat("/");
            s = subsby.toString(p, f, st);
        }
        return s;
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public int Type() {
        return 1;
    }


    /**
     *  Description of the Method
     *
     *@param  t2  Description of the Parameter
     *@return     Description of the Return Value
     */
    public boolean Unify(Term t2) {
        // Unifikation der Variable this mit Term t2
        if (this.subsby != null) {
            // first is a variable and is already substituted
            return t2.Unify(this.subsby);
        }
        else if (t2 instanceof Variable) {
            // second is also a variable
            if (((Variable) t2).subsby != null) {
                // second is substituted
                return this.Unify(((Variable) t2).subsby);
            }
            else {
                // both are variables which are not substituted
                ((Variable) t2).symbol = symbol;
            }
            return true;
        }
        else {
            // second is no variable
            if (!t2.occurs(this)) {
                this.subsby = t2;
                return true;
            }
            else {
                // first variable occurs in second term
                return false;
            }
        }
    }
}
