package org.semanticweb.triple.language;

import java.util.*;
import org.semanticweb.glp.*;


public class HornAtom extends Atom {

  String predicateSymbol;
  ArrayList args = new ArrayList(); // of Term

  public void putPredicateSymbol(String predicateSymbol) { 
    this.predicateSymbol = predicateSymbol; 
  }

  public void putArgs(Term term) { args.add(term); }
  public void putArgs(String name) { args.add(new Literal(name)); }

  public String getPredicateSymbol() { return predicateSymbol; }
  public ArrayList getArgs() { return args; }

  public String toString(Syntax syntax) {
    StringBuffer buffer = new StringBuffer();
    buffer.append(syntax.getSymbol(predicateSymbol));
    buffer.append("(");
    if (args != null && args.size() > 0) {
      for (Iterator termIterator = args.iterator();
	   termIterator.hasNext();) {
	Term term = (Term)termIterator.next();
	buffer.append(term.toString(syntax));
	if (termIterator.hasNext())
	  buffer.append(", ");
      }
    }
    buffer.append(")");
    appendModel(syntax, buffer);
    return buffer.toString();
  }
  
  public GLogicFormula toGLP(GLPTerm model) {
    // p(a1, ...) -> ..., true(p(a1', ...), model) for normal predicates
    // special treatments for builtins, EXTERNAL etc.
    // this -> triple struct
    ArrayList additionalGlpFormulae = new ArrayList();
    model = overrideModel(model, this.model, additionalGlpFormulae);
    if (predicateSymbol.equalsIgnoreCase("EXTERNAL")) // external program call
      return toGLPExternalFormula(model, additionalGlpFormulae);
    // else builtins ... !!!
    else
      return toGLPTrueFormula(model, additionalGlpFormulae);
  }
    
  public GLogicFormula toGLPTrueFormula(GLPTerm model,
      ArrayList additionalGlpFormulae) {
    ArrayList structargs = new ArrayList(3);
    for (Iterator argIterator = args.iterator();
	 argIterator.hasNext();) {
      Term arg = (Term)argIterator.next();
      GLPTerm newArg = (GLPTerm)arg.toGLP(model, additionalGlpFormulae);
      structargs.add(newArg);
    }
    GLPTermFunction struct = 
      new GLPTermFunction(predicateSymbol, structargs);
    ArrayList trueargs = new ArrayList(2);
    trueargs.add(struct);
    trueargs.add(model);
    GLPAtom trueAtom = new GLPAtom("true", trueargs);
    GLogicFormula conjunction =
      And.makeGLPConjunction(additionalGlpFormulae, trueAtom);
    return conjunction;
  }
  
  public GLogicFormula toGLPExternalFormula(GLPTerm model,
      ArrayList additionalGlpFormulae) {
    // EXTERNAL(program(model2))@model -> external_model(program, model2, model)
    if (args.size() != 1) {
      System.err.println("Warning: not a correct EXTERNAL statement: " +
        this.toString());
      // we probably should throw an exception ...
      return toGLPTrueFormula(model, additionalGlpFormulae);
    }
    Term programTerm = (Term)args.get(0); // program(model2)
    if (!(programTerm instanceof Structure)) {
      System.err.println("Warning: not a correct EXTERNAL statement: " +
        this.toString());
      return toGLPTrueFormula(model, additionalGlpFormulae);
    }
    String programName = ((Structure)programTerm).getFunctor();
    GLPTermConstant program = new GLPTermConstant(programName);
    ArrayList programArgs = ((Structure)programTerm).getArgs();
    if (programArgs.size() != 1) {
      System.err.println("Warning: not a correct EXTERNAL statement: " +
        this.toString());
      return toGLPTrueFormula(model, additionalGlpFormulae);
    }
    Term model2term = (Term)programArgs.get(0);
    GLPTerm model2 = model2term.toGLP(model, additionalGlpFormulae);
    ArrayList extargs = new ArrayList(3);
    extargs.add(program);
    extargs.add(model2);
    extargs.add(model);
    GLPAtom extAtom = new GLPAtom("external_model", extargs);
    GLogicFormula conjunction = // does this make sense here?
      And.makeGLPConjunction(additionalGlpFormulae, extAtom);
    return conjunction;
  }
  
}
